;=====================================================================
;                           PV-WAVE ARL
;
;                 PV-Wave Advanced Rendering Library
;                        11/90   D. Carr
;
;             (c) Copyright Precision Visuals, Inc., 1990
;                     Boulder, Colorado, U.S.A.
;
;                        All Rights Reserved.
;
;  This software is confidential information which is proprietary to
;  and a trade secret of Precision Visuals, Inc. Use, duplication or
;  disclosure is subject to the terms of an appropriate license
;  agreement.
;  ------------------------------------------------------------------
;  Use, duplication, or disclosure by the Government is subject to
;  restrictions set forth in paragraph (b)(3)(B) of the Rights in
;  Technical Data and Computer Software clause in DAR 7-104.9(a).
;
;  Contractor : Precision Visuals, Inc., Boulder, Colorado, U.S.A.
;  -------------------------------------------------------------------
;
;  Program Overview : Center_View
;  Center_View sets the system 3-D viewing transformation and system
;  DATA to NORMAL coordinate conversion factors so that data will be
;  displayed in the center of the current window.   The correct aspect
;  ratio of the data will be preserved even if the plot window is not
;  square.
;
;  CAUTION : This procedure sets the system variables !P.T , !P.T3d ,
;            !X.S , !Y.S , and !Z.S .
;            
;
;  SEE ALSO :  Set_View3d
;
;  MANDATORY INPUTS : None
;
;  OPTIONAL INPUTS :
;
;  xr :             This is a two element vector with xr(0) containing
;                   the minimum X value in the data to be plotted, and
;                   xr(1) containing the maximum value.   The default
;                   is [-1.0, 1.0] .
;
;  yr :             This is the same as xr except this contains the
;                   Y values.
;
;  zr :             This is the same as xr except this contains the
;                   Z values.
;
;  ax :             The angle to rotate the data around the X axis.
;                   The default is -60.0 .
;                   NOTE : The data is rotated az degrees about the
;                          Z axis first, ay degrees about the Y axis
;                          second, and ax degrees about the X axis last.
;
;  ay :             The angle to rotate the data around the Z axis.
;                   The default is 0.0 .
;
;  az :             The angle to rotate the data around the Z axis.
;                   The default is 30.0 .
;
;  winx :           The X size of the PV-WAVE plot window in DEVICE
;                   coordinates.   The default is 640 .
;
;  winy :           The Y size of the PV-WAVE plot window in DEVICE
;                   coordinates.   The default is 512 .
;
;  zoom :           The magnification factor.   If zoom contains one
;                   element, then the view will be zoomed equally in
;                   the X, Y, and Z dimensions.   If zoom contains
;                   3 elements, then the view will be scaled by
;                   zoom(0) in the X direction, zoom(1) in the Y
;                   direction, and zoom(2) in the Z direction.
;                   The default is [0.7, 0.7, 0.7] .
;
;  persp :          The perspective projection distance.   The default
;                   is NO perspective projection.
;
;=====================================================================
;
PRO Center_View, xr=xr, yr=yr, zr=zr, ax=ax, ay=ay, az=az, $
    winx=winx, winy=winy, zoom=zoom, persp=persp

x_range = [-1.0, 1.0]
y_range = [-1.0, 1.0]
z_range = [-1.0, 1.0]
IF (N_Elements(xr) GE 2) THEN x_range = Float(xr(0:1))
IF (N_Elements(yr) GE 2) THEN y_range = Float(yr(0:1))
IF (N_Elements(zr) GE 2) THEN z_range = Float(zr(0:1))
ang_x = -60.0
ang_y =   0.0
ang_z =  30.0
IF (N_Elements(ax) GT 0) THEN ang_x = Float(ax(0))
IF (N_Elements(ay) GT 0) THEN ang_y = Float(ay(0))
IF (N_Elements(az) GT 0) THEN ang_z = Float(az(0))
window_x = 640.0
window_y = 512.0
IF (N_Elements(winx) GT 0) THEN window_x = Float(winx(0))
IF (N_Elements(winy) GT 0) THEN window_y = Float(winy(0))
magnify = [0.7, 0.7, 0.7]
IF (N_Elements(zoom) EQ 1) THEN magnify(*) = Float(zoom(0))
IF (N_Elements(zoom) EQ 2) THEN BEGIN
   magnify(0) = Float(zoom(0))
   magnify(1) = Float(zoom(1))
ENDIF
IF (N_Elements(zoom) GE 3) THEN magnify(*) = Float(zoom(0:2))
perspective = 0.0
IF (N_Elements(persp) GT 0) THEN perspective = Float(persp(0))

x_wide = x_range(1) - x_range(0)
y_wide = y_range(1) - y_range(0)
z_wide = z_range(1) - z_range(0)
x_half = (x_range(0) + x_range(1)) / 2.0
y_half = (y_range(0) + y_range(1)) / 2.0
z_half = (z_range(0) + z_range(1)) / 2.0
max_range = Max([x_wide, y_wide, z_wide])
mid_range = max_range / 2.0

!X.S = [(mid_range - x_half), 1.0] / max_range
!Y.S = [(mid_range - y_half), 1.0] / max_range
!Z.S = [(mid_range - z_half), 1.0] / max_range
aspx = 1.0
aspy = 1.0
IF (window_x GT window_y) THEN BEGIN
   aspx = window_y / window_x
   aspy = 1.0
ENDIF
IF (window_x LT window_y) THEN BEGIN
   aspx = 1.0
   aspy = window_x / window_y
ENDIF

T3d, /Reset
T3d, Translate=[-0.5, -0.5, -0.5]
T3d, Scale=magnify
T3d, Rotate=[0.0, 0.0, ang_z]
T3d, Rotate=[0.0, ang_y, 0.0]
T3d, Rotate=[ang_x, 0.0, 0.0]
IF (N_Elements(persp) GT 0) THEN T3d, perspective=perspective
T3d, Scale=[aspx, aspy, 1.0]
T3d, Translate=[0.5, 0.5, 0.5]

!P.T3d = 1

END
