;=====================================================================
;                           PV-WAVE ARL
;
;                 PV-Wave Advanced Rendering Library
;                        11/90   D. Carr
;
;             (c) Copyright Precision Visuals, Inc., 1990
;                     Boulder, Colorado, U.S.A.
;
;                        All Rights Reserved.
;
;  This software is confidential information which is proprietary to
;  and a trade secret of Precision Visuals, Inc. Use, duplication or
;  disclosure is subject to the terms of an appropriate license
;  agreement.
;  ------------------------------------------------------------------
;  Use, duplication, or disclosure by the Government is subject to
;  restrictions set forth in paragraph (b)(3)(B) of the Rights in
;  Technical Data and Computer Software clause in DAR 7-104.9(a).
;
;  Contractor : Precision Visuals, Inc., Boulder, Colorado, U.S.A.
;  -------------------------------------------------------------------
;
;  Program Overview : FUNCTION Conv_From_Rect
;  Conv_From_Rect converts coordinates from rectangular to polar,
;  cylindrical or spherical coordinates.   If the "polar" keyword is
;  present and non-zero then a float(2, n) array is returned with
;  (0, *) containing the angles and (1, *) containing the radii.
;  If the "cylin" keyword is present and non-zero, then a float(3, n)
;  array is returned with (0, *) containing the angles, (1, *)
;  containing the radii, and (2, *) containing the Z values.
;  Otherwise, spherical coordinates are returned with (0, *)
;  containing the longitude, (1, *) containing the latitude,
;  and (2, *) containig the radii.
;
;  SEE ALSO :  Conv_To_Rect
;
;  MANDATORY INPUTS :
;
;  vec1 :           A one dimensional array containing the X
;                   rectangular coordinates.
;
;  vec2 :           A one dimensional array containing the Y
;                   rectangular coordinates.
;
;  vec3 :           A one dimensional array containing the Z
;                   rectangular coordinates.   For polar coordinates,
;                   set vec3 to the scalar value 0 .
;
;  OPTIONAL COORDINATES :
;
;  degrees :        If "degrees" is present and non-zero then the
;                   returned coordinates will be in degrees in place
;                   of radians.
;
;  polar :          Specifies that polar coordinates are to be
;                   returned.
;
;  cylin :          Specifies that cylindrical coordinates are to be
;                   returned.
;                  
;=====================================================================
;
FUNCTION Conv_From_Rect, vec1, vec2, vec3, degrees=degrees, polar=polar, cylin=cylin

vec1 = Float(vec1)
vec2 = Float(vec2)
vec3 = Float(vec3)

IF (N_Elements(degrees) EQ 0) THEN degrees = 0
IF (N_Elements(polar) EQ 0) THEN polar = 0
IF (N_Elements(cylin) EQ 0) THEN cylin = 0
IF ((polar(0) EQ 1) AND (cylin(0) EQ 1)) THEN BEGIN
   Print, "Polar and Cylin keywords can't both be present."
   STOP
ENDIF

sphere = 0
IF ((polar NE 1) AND (cylin NE 1)) THEN sphere = 1

n_points = (N_Elements(vec1))
IF (n_points EQ 0) THEN BEGIN
   Print, "No points to convert."
   STOP
ENDIF

IF (N_Elements(vec2) NE n_points) THEN BEGIN
   Print, "vec1 and vec2 arrays are not the same size."
   STOP
ENDIF

IF ((polar(0) NE 1) AND (N_Elements(vec3) NE n_points)) THEN BEGIN
   Print, "vec1 and vec3 arrays are not the same size."
   STOP
ENDIF

IF (sphere EQ 1) THEN BEGIN
   points = Fltarr(3, n_points)
   points(0, *) = Atan(vec2, vec1)
   points(1, *) = Atan(vec3, Sqrt((vec1 * vec1) + (vec2 * vec2)))
   points(2, *) = Sqrt((vec1 * vec1) + (vec2 * vec2) +(vec3 * vec3))
ENDIF

IF (cylin(0) EQ 1) THEN BEGIN
   points = Fltarr(3, n_points)
   points(0, *) = Atan(vec2, vec1)
   points(1, *) = Sqrt((vec1 * vec1) + (vec2 * vec2))
   points(2, *) = vec3
ENDIF

IF (polar(0) EQ 1) THEN BEGIN
   points = Fltarr(2, n_points)
   points(0, *) = Atan(vec2, vec1)
   points(1, *) = Sqrt((vec1 * vec1) + (vec2 * vec2))
ENDIF

RETURN, points
END
