;=====================================================================
;                           PV-WAVE ARL
;
;                 PV-Wave Advanced Rendering Library
;                        11/90   D. Carr
;
;             (c) Copyright Precision Visuals, Inc., 1990
;                     Boulder, Colorado, U.S.A.
;
;                        All Rights Reserved.
;
;  This software is confidential information which is proprietary to
;  and a trade secret of Precision Visuals, Inc. Use, duplication or
;  disclosure is subject to the terms of an appropriate license
;  agreement.
;  ------------------------------------------------------------------
;  Use, duplication, or disclosure by the Government is subject to
;  restrictions set forth in paragraph (b)(3)(B) of the Rights in
;  Technical Data and Computer Software clause in DAR 7-104.9(a).
;
;  Contractor : Precision Visuals, Inc., Boulder, Colorado, U.S.A.
;  -------------------------------------------------------------------
;
;  Program Overview : FUNCTION Grid_3d
;
;  Grid_3d takes random X-Y-Z coordinates and returns a gridded
;  2 dimensional array containing Z values.   This gridded array is
;  suitable for use with the "Surface", "Tv", and "Contour" functions.
;  Grid_3d uses a weighted neighborhood averaging technique to
;  interpolate missing data values.
;
;  SEE ALSO :  Grid_2d  Grid_4d  Grid_Sphere
;
;  MANDATORY INPUTS :
;
;  points :         A (3, n) array containing random X-Y-Z points to
;                   be gridded.
;
;  grid_x :         The X dimension of the grid.   The X values are
;                   scaled to fit this dimension.
;
;  grid_y :         The Y dimension of the grid.   The Y values are
;                   scaled to fit this dimension.
;
;  OPTIONAL INPUTS (Keywords) :
;
;  order :          The order of the weighing function to use for
;                   neighborhood averaging.   The default is 2 .
;                   Points are weighted by the function :
;                      w = 1.0 / (dist^order)
;                   where dist is the distance to the point. 
;
;=====================================================================
;
FUNCTION Grid_3d, points, grid_x, grid_y, order=order

sp = Size(points)
IF (sp(0) NE 2) THEN BEGIN
   Print, "Invalid points array."
   STOP
ENDIF
IF (sp(1) NE 3) THEN BEGIN
   Print, "Invalid points array."
   STOP
ENDIF
IF (grid_x(0) LT 2) THEN BEGIN
   Print, "X dimension of grid must be 2 or greater."
   STOP
ENDIF
IF (grid_y(0) LT 2) THEN BEGIN
   Print, "Y dimension of grid must be 2 or greater."
   STOP
ENDIF
 
p_order = 2.0 
IF (N_Elements(order) GT 0) THEN p_order = Float(order(0)) > 0.0
 
points = Float(points)
n_points = sp(2)

grid_x = Fix(grid_x(0))
grid_y = Fix(grid_y(0))

lox = Min(points(0, *))
hix = Max(points(0, *))
loy = Min(points(1, *))
hiy = Max(points(1, *))
xrange = hix - lox
yrange = hiy - loy

IF ((xrange LE 0.0) OR (yrange LE 0.0)) THEN BEGIN
   Print, "Unable to grid points."
   Print, "All points have the same X or Y value."
   STOP
ENDIF

z_val = Fltarr(grid_x, grid_y)

xinc = xrange / Float(grid_x)
yinc = yrange / Float(grid_y)

xs = Fix((points(0, *) - lox) / xinc) < (grid_x - 1)
ys = Fix((points(1, *) - loy) / yinc) < (grid_y - 1)

grid_xm1 = grid_x - 1
grid_ym1 = grid_y - 1
FOR j=0, grid_ym1 DO BEGIN
   dy = (Float(ys - j))^2
   FOR i=0, grid_xm1 DO BEGIN
      dx = (Float(xs - i))^2
      distances = (dx + dy)^(0.5 * p_order)
      ind = Where(distances EQ 0.0)
      IF (ind(0) EQ (-1)) THEN BEGIN
         z_val(i, j) = Total(points(2, *) / distances) / Total(1.0 / distances)
      ENDIF ELSE BEGIN
         z_val(i, j) = AVG(points(2, ind))
      ENDELSE
   ENDFOR
ENDFOR

RETURN, z_val
END
