;=====================================================================
;                           PV-WAVE ARL
;
;                 PV-Wave Advanced Rendering Library
;                        11/90   D. Carr
;
;             (c) Copyright Precision Visuals, Inc., 1990
;                     Boulder, Colorado, U.S.A.
;
;                        All Rights Reserved.
;
;  This software is confidential information which is proprietary to
;  and a trade secret of Precision Visuals, Inc. Use, duplication or
;  disclosure is subject to the terms of an appropriate license
;  agreement.
;  ------------------------------------------------------------------
;  Use, duplication, or disclosure by the Government is subject to
;  restrictions set forth in paragraph (b)(3)(B) of the Rights in
;  Technical Data and Computer Software clause in DAR 7-104.9(a).
;
;  Contractor : Precision Visuals, Inc., Boulder, Colorado, U.S.A.
;  -------------------------------------------------------------------
;
;  Program Overview : FUNCTION Grid_Sphere
;
;  Grid_Sphere takes random longitude, latitude, and radius
;  coordinates and returns a gridded 2 dimensional array containing
;  radius values.   This gridded array is suitable for use with the
;  "Poly_Sphere" function.   Grid_Sphere uses a weighted neighborhood
;  averaging technique to interpolate missing data values.
;
;  SEE ALSO :  Grid_2d  Grid_3d  Grid_4d
;
;  MANDATORY INPUTS :
;
;  points :         A (3, n) array containing random longitude,
;                   latitude, and radii coordinates.
;
;  grid_x :         The X dimension of the grid.   The X values are
;                   scaled to fit this dimension.
;
;  grid_y :         The Y dimension of the grid.   The Y values are
;                   scaled to fit this dimension.
;
;  OPTIONAL INPUTS (Keywords) :
;
;  degrees :        If "degrees" is present and non-zero then the
;                   input coordinates are in degrees instead of
;                   radians.
;
;  order :          The order of the weighing function to use for
;                   neighborhood averaging.   The default is 2 .
;                   Points are weighted by the function :
;                      w = 1.0 / (dist^order)
;                   where dist is the distance to the point.
;
;
;=====================================================================
;
FUNCTION Grid_Sphere, points, grid_x, grid_y, degrees=degrees, $
                      order=order

sp = Size(points)
n_points = sp(2)
points = Float(points)

IF (sp(0) NE 2) THEN BEGIN
   Print, "Invalid points array."
   STOP
ENDIF
IF (sp(1) NE 3) THEN BEGIN
   Print, "Invalid points array."
   STOP
ENDIF
IF (grid_x(0) LT 2) THEN BEGIN
   Print, "X dimension of grid must be 2 or greater."
   STOP
ENDIF
IF (grid_y(0) LT 2) THEN BEGIN
   Print, "Y dimension of grid must be 2 or greater."
   STOP
ENDIF
IF (N_Elements(degrees) GT 0) THEN BEGIN
   IF (degrees(0) NE 0) THEN BEGIN
      points(0, *) = points(0, *) * !PI / 180.0
      points(1, *) = points(1, *) * !PI / 180.0
   ENDIF
ENDIF
IF ((Max(points(0, *)) GT (2.0 * !PI)) OR $
    (Min(points(0, *)) LT 0.0)) THEN BEGIN
   Print, "Longitude values must be between 0 and 2*PI"
   STOP
ENDIF
IF ((Max(points(1, *)) GT !PI) OR $
    (Min(points(1, *)) LT -!PI)) THEN BEGIN
   Print, "Latitude values must be between -PI and +PI"
   STOP
ENDIF

p_order = 2.0
IF (N_Elements(order) GT 0) THEN p_order = Float(order(0)) > 0.0

grid_x = Fix(grid_x(0))
grid_y = Fix(grid_y(0))

lox = 0.0
hix = 2.0 * !PI
loy = (-!PI)
hiy = !PI
xrange = hix - lox
yrange = hiy - loy

rad_val = Fltarr(grid_x, grid_y)

xinc = xrange / Float(grid_x - 1)
yinc = yrange / Float(grid_y - 1)

xs = (Float(Fix((points(0, *) - lox) / xinc))) * 2.0 * !PI / Float(grid_x)
ys = ((Float(Fix((points(1, *) - loy) / yinc))) * !PI / Float(grid_y)) - (!PI / 2.0)
xr = Cos(xs) * Cos(ys)
yr = Sin(xs) * Cos(ys)
zr = Sin(ys)

grid_xm1 = grid_x - 1
grid_ym1 = grid_y - 1
FOR j=0, grid_ym1 DO BEGIN
   FOR i=0, grid_xm1 DO BEGIN
      xi = Float(i) * 2.0 * !PI / Float(grid_x - 1)
      yj = (Float(j) * !PI / Float(grid_y - 1)) - (!PI / 2.0)
      cx = Cos(xi) * Cos(yj)
      cy = Sin(xi) * Cos(yj)
      cz = Sin(yj)
      dx = (cx - xr) * (cx - xr)
      dy = (cy - yr) * (cy - yr)
      dz = (cz - zr) * (cz - zr)
      distances = (dx + dy + dz)^(0.5 * p_order)
      ind = Where(distances EQ 0.0)
      IF (ind(0) EQ (-1)) THEN BEGIN
         rad_val(i, j) = Total(points(2, *) / distances) / Total(1.0 / distances)
      ENDIF ELSE BEGIN
         rad_val(i, j) = AVG(points(2, ind))
      ENDELSE
   ENDFOR
ENDFOR

RETURN, rad_val
END
