;=====================================================================
;                           PV-WAVE ARL
;
;                 PV-Wave Advanced Rendering Library
;                        11/90   D. Carr
;
;             (c) Copyright Precision Visuals, Inc., 1990
;                     Boulder, Colorado, U.S.A.
;
;                        All Rights Reserved.
;
;  This software is confidential information which is proprietary to
;  and a trade secret of Precision Visuals, Inc. Use, duplication or
;  disclosure is subject to the terms of an appropriate license
;  agreement.
;  ------------------------------------------------------------------
;  Use, duplication, or disclosure by the Government is subject to
;  restrictions set forth in paragraph (b)(3)(B) of the Rights in
;  Technical Data and Computer Software clause in DAR 7-104.9(a).
;
;  Contractor : Precision Visuals, Inc., Boulder, Colorado, U.S.A.
;  -------------------------------------------------------------------
;
;  Program Overview : PROCEDURE Poly_Merge
;  Poly_Merge merges two vertex lists and polygon lists together so
;  that they may be rendered in a single pass by "Polyshade" or
;  "Poly_Plot".
;
;  MANDATORY INPUTS :
;
;  vertex_list1 :      The first vertex list.   See the documentation
;                      on the "Poly_Plot" routine for more information
;                      about vertex lists and polygon lists.
;
;  vertex_list2 :      The second vertex list.
;
;  polygon_list1 :     The first polygon list.
;
;  polygon_list2 :     The second polygon list.
;
;  vert :              On input, vert may be undefined.   On return,
;                      vert contains vertex_list1 and vertex_list2
;                      merged together.
;
;  poly :              On input, poly may be undefined.   On return,
;                      poly contains polygon_list1 and polygon_list2
;                      merged together and modified so that it is
;                      compatible with vert.
;
;  pg_num :            On input, pg_num may be undefined.   On return,
;                      pg_num is the total number of polygons in the
;                      merged list.
;
;  OPTIONAL INPUTS (Keywords) :
;
;  fill1 :             A vector containing the fill colors for the
;                      first polygon list.
;
;  fill2 :             A vector containing the fill colors for the
;                      second polygon list.
;
;  edge1 :             A vector containing the edge colors for the
;                      first polygon list.
;
;  edge2 :             A vector containing the edge colors for the
;                      second polygon list.
;
;  opaque1 :           A vector containing the translucency factors
;                      for the first polygon list.
;
;  opaque2 :           A vector containing the translucency factors
;                      for the second polygon list.
;
;  fill_list :         On input, fill_list may be undefined.
;                      On return, fill_list contains the fill colors
;                      for the merged list.
;
;  edge_list :         On input, edge_list may be undefined.
;                      On return, edge_list contains the edge colors
;                      for the merged list.
;
;  opaque_list :       On input, opaque_list may be undefined.
;                      On return, opaque_list contains the
;                      translucency factors for the merged list.
;
;=====================================================================
;
PRO Poly_Merge, vertex_list1, vertex_list2, polygon_list1, $
                polygon_list2, vert, poly, pg_num, fill1=fill1, $
                fill2=fill2, edge1=edge1, edge2=edge2, $
                opaque1=opaque1, opaque2=opaque2, $
                fill_list=fill_list, edge_list=edge_list, $
                opaque_list=opaque_list

L1 = Long(1)

sv = Size(vertex_list1)
IF (sv(0) NE 2) THEN BEGIN
   Print, "Invalid vertex list."
   STOP
ENDIF
IF (sv(1) NE 3) THEN BEGIN
   Print, "Invalid vertex list."
   STOP
ENDIF
IF ((Max(polygon_list1) GE sv(2)) OR (Min(polygon_list1) LT 0))THEN BEGIN
   Print, "Invalid polygon list."
   STOP
ENDIF
sv1 = sv(2)
sv = Size(vertex_list2)
IF (sv(0) NE 2) THEN BEGIN
   Print, "Invalid vertex list."
   STOP
ENDIF
IF (sv(1) NE 3) THEN BEGIN
   Print, "Invalid vertex list."
   STOP
ENDIF
IF ((Max(polygon_list2) GE sv(2)) OR (Min(polygon_list2) LT 0))THEN BEGIN
   Print, "Invalid polygon list."
   STOP
ENDIF
sv2 = sv(2)
vert = Fltarr(3, (sv1 + sv2))

sp = Size(polygon_list1)
sp1 = sp(1)
sp = Size(polygon_list2)
sp2 = sp(1)
poly = Lonarr(sp1 + sp2)

vert(0:2, 0:(sv1-1)) = Float(vertex_list1(0:2, *))
vert(0:2, sv1:*) = Float(vertex_list2(0:2, *))
poly(0:(sp1-1)) = Long(polygon_list1)
poly(sp1:*) = Long(polygon_list2)

poly_max = sp1 + sp2
poly_pt = sp1
WHILE (poly_pt LT poly_max) DO BEGIN
   FOR i=L1, poly(poly_pt) DO poly(poly_pt + i) = poly(poly_pt + i) + sv1
   poly_pt = poly_pt + poly(poly_pt) + L1
ENDWHILE

pg_num1 = Poly_Count(Polygon_list1)
pg_num2 = Poly_Count(Polygon_list2)
pg_num = pg_num1 + pg_num2

IF ((N_Elements(fill1) GT 0) AND (N_Elements(fill2) GT 0)) THEN BEGIN
   IF ((N_Elements(fill1) NE pg_num1) OR (N_Elements(fill2) NE pg_num2)) THEN BEGIN
      Print, "Invalid fill colors array."
      STOP
   ENDIF
   fill_list = Intarr(pg_num)
   fill_list(0:(pg_num1-1)) = fill1(*)
   fill_list(pg_num1:*) = fill2(*)
ENDIF

IF ((N_Elements(edge1) GT 0) AND (N_Elements(edge2) GT 0)) THEN BEGIN
   IF ((N_Elements(edge1) NE pg_num1) OR (N_Elements(edge2) NE pg_num2)) THEN BEGIN
      Print, "Invalid edge colors array."
      STOP
   ENDIF
   edge_list = Intarr(pg_num)
   edge_list(0:(pg_num1-1)) = edge1(*)
   edge_list(pg_num1:*) = edge2(*)
ENDIF

IF ((N_Elements(opaque1) GT 0) AND (N_Elements(opaque2) GT 0)) THEN BEGIN
   IF ((N_Elements(opaque1) NE pg_num1) OR (N_Elements(opaque2) NE pg_num2)) THEN BEGIN
      Print, "Invalid opaque array."
      STOP
   ENDIF
   opaque_list = Intarr(pg_num)
   opaque_list(0:(pg_num1-1)) = opaque1(*)
   opaque_list(pg_num1:*) = opaque2(*)
ENDIF

RETURN
END
