;=====================================================================
;                           PV-WAVE ARL
;
;                 PV-Wave Advanced Rendering Library
;                        11/90   D. Carr
;
;             (c) Copyright Precision Visuals, Inc., 1990
;                     Boulder, Colorado, U.S.A.
;
;                        All Rights Reserved.
;
;  This software is confidential information which is proprietary to
;  and a trade secret of Precision Visuals, Inc. Use, duplication or
;  disclosure is subject to the terms of an appropriate license
;  agreement.
;  ------------------------------------------------------------------
;  Use, duplication, or disclosure by the Government is subject to
;  restrictions set forth in paragraph (b)(3)(B) of the Rights in
;  Technical Data and Computer Software clause in DAR 7-104.9(a).
;
;  Contractor : Precision Visuals, Inc., Boulder, Colorado, U.S.A.
;  -------------------------------------------------------------------
;
;  Program Overview : PROCEDURE Set_View3d
;
;  Set_View3d creates a 3-D viewing transformation based on a 3-D
;  viewpoint and a view direction.
;
;  NOTE : Set_View3d will create a view transformation that will
;         preserve the correct aspect ratio of the data even if
;         the plot window is non-square.
;
;  WARNING : Set_view3d changes the system viewing matrix !P.T as
;            well as the system DATA to NORMAL coordinate conversion
;            factors !X.S , !Y.S , and !Z.S .
;
;  SEE ALSO :  Center_View
;
;  MANDATORY INPUTS :
;
;  viewpoint :      A 3 element vector containing the point from
;                   which to view the data in DATA coordinates.
;
;  viewvector :     A 3 element vector containing the direction to
;                   look.
;
;  perspective :    The perspective projection distance.   The smaller
;                   the projection distance, the more "severe" the
;                   projection will be.   The larger the projection
;                   distance, the more "isometric" the projection will
;                   be.   To prevent a perspective projection, set
;                   perspective to 0 (or less than zero).
;
;  izoom :          The magnification factor for the projection.
;
;  viewup :         A 2 element vector containing the final 2-D
;                   view up vector.   For a "right-side-up" view,
;                   set this parameter to [0.0, 1.0] .
;
;  viewcenter :     A 2 element vector containing the window location
;                   on which to place the viewpoint.   The viewcenter
;                   parameter is in NORMAL coordinates and is usually
;                   set to [0.5, 0.5] .
;
;  winx :           The X dimension of the plot window in DEVICE
;                   coordinates.   Usually, winx is set to the X size
;                   of the current PV-WAVE window.
;
;  winy :           The Y dimension of the plot window in DEVICE
;                   coordinates.   Usually, winy is set to the Y size
;                   of the current PV-WAVE window.
;
;  xr :             A 2 element vector containing the minimum and
;                   maximum X values found in the data to be plotted.
;                   The minimum value should be in xr(0) and the
;                   maximum value in xr(1) .
;
;  yr :             This is the same as xr except that it specifies
;                   Y values.
;
;  zr :             This is the same as xr except that it specifies
;                   Z values.
;
;=====================================================================
;
PRO Set_View3D, viewpoint, viewvector, perspective, izoom, viewup, $
                viewcenter, winx, winy, xr, yr, zr

IF (N_Elements(viewpoint) NE 3) THEN BEGIN
   Print, "viewpoint must contain 3 elements."
   STOP
ENDIF

IF (N_Elements(viewvector) NE 3) THEN BEGIN
   Print, "viewvector must contain 3 elements."
   STOP
ENDIF

IF (N_Elements(viewup) NE 2) THEN BEGIN
   Print, "viewup must contain 2 elements."
   STOP
ENDIF

IF (N_Elements(viewcenter) NE 2) THEN BEGIN
   Print, "viewcenter must contain 2 elements."
   STOP
ENDIF

IF (N_Elements(xr) NE 2) THEN BEGIN
   Print, "X range must contain 2 elements."
   STOP
ENDIF

IF (N_Elements(yr) NE 2) THEN BEGIN
   Print, "Y range must contain 2 elements."
   STOP
ENDIF

IF (N_Elements(zr) NE 2) THEN BEGIN
   Print, "Z range must contain 2 elements."
   STOP
ENDIF

xwide = Float(xr(1) - xr(0))
xmid = Float(xr(0) + xr(1)) / 2.0

ywide = Float(yr(1) - yr(0))
ymid = Float(yr(0) + yr(1)) / 2.0

zwide = Float(zr(1) - zr(0))
zmid = Float(zr(0) + zr(1)) / 2.0

width = Max([xwide, ywide, zwide])
width2 = width / 2.0

!X.S = [(width2 - xmid), 1.0] / width
!Y.S = [(width2 - ymid), 1.0] / width
!Z.S = [(width2 - zmid), 1.0] / width

viewx = -(!X.S(0) + (!X.S(1) * Float(viewpoint(0))))
viewy = -(!Y.S(0) + (!Y.S(1) * Float(viewpoint(1))))
viewz = -(!Z.S(0) + (!Z.S(1) * Float(viewpoint(2))))

viewvecx = Float(viewvector(0))
viewvecy = Float(viewvector(1))
viewvecz = Float(viewvector(2))

;*****   Calculate the azimuth rotation.
IF (Float(viewvecy) EQ 0.0) THEN BEGIN
   IF (Float(viewvecx) EQ 0.0) THEN rotz = 0.0
   IF (Float(viewvecx) GT 0.0) THEN rotz = 90.0
   IF (Float(viewvecx) LT 0.0) THEN rotz = -90.0
ENDIF ELSE BEGIN
   rotz = Float(Atan(Float(viewvecx), $
                     Float(viewvecy))) * Float(!Radeg)
ENDELSE

;*****   Calculate the elevation rotation.
len = Sqrt((Float(viewvecx)^2) + (Float(viewvecy)^2))
IF (Float(viewvecz) EQ 0.0) THEN BEGIN
   rotx = -90.0
ENDIF ELSE BEGIN
   rotx = -(Float(Atan(len, Float(-viewvecz))) * Float(!Radeg))
ENDELSE

;*****   Calculate the view up rotation.
IF (Float(viewup(1)) EQ 0.0) THEN BEGIN
   IF (Float(viewup(0)) EQ 0.0) THEN rotu = 0.0
   IF (Float(viewup(0)) GT 0.0) THEN rotu = 90.0
   IF (Float(viewup(0)) LT 0.0) THEN rotu = -90.0
ENDIF ELSE BEGIN
   rotu = Float(Atan(Float(viewup(0)), Float(viewup(1)))) * Float(!Radeg)
ENDELSE

;*****   Calculate the aspect scale factors.
aspect = 1.0
IF (Float(winy(0)) GT 0.0) THEN aspect = Float(winx(0)) / Float(winy(0))
IF (aspect GE 1.0) THEN BEGIN
   aspx = 1.0
   aspy = aspect
ENDIF ELSE BEGIN
   IF (aspect GT 0.0) THEN BEGIN
      aspx = 1.0 / aspect
      aspy = 1.0
   ENDIF ELSE BEGIN
      aspx = 1.0
      aspy = 1.0
   ENDELSE
ENDELSE

;*****   Start building view transformation.   ******

T3d, /Reset
T3d, Translate = [viewx, viewy, viewz]
T3d, Rotate = [0.0, 0.0, rotz]
T3d, Rotate = [rotx, 0.0, 0.0]
IF (perspective(0) GT 0.0) THEN T3d, perspective = Float(perspective(0))
T3d, Scale = Float([izoom(0), izoom(0), 1.0])
T3d, Rotate = [0.0, 0.0, rotu]
T3d, Scale = [aspx, aspy, 1.0]
T3d, Translate = [Float(viewcenter(0)), Float(viewcenter(1)), 0.0]

RETURN
END
