;=====================================================================
;                           PV-WAVE ARL
;
;                 PV-Wave Advanced Rendering Library
;                        11/90   D. Carr
;
;             (c) Copyright Precision Visuals, Inc., 1990
;                     Boulder, Colorado, U.S.A.
;
;                        All Rights Reserved.
;
;  This software is confidential information which is proprietary to
;  and a trade secret of Precision Visuals, Inc. Use, duplication or
;  disclosure is subject to the terms of an appropriate license
;  agreement.
;  ------------------------------------------------------------------
;  Use, duplication, or disclosure by the Government is subject to
;  restrictions set forth in paragraph (b)(3)(B) of the Rights in
;  Technical Data and Computer Software clause in DAR 7-104.9(a).
;
;  Contractor : Precision Visuals, Inc., Boulder, Colorado, U.S.A.
;  -------------------------------------------------------------------
;
;  Program Overview : PROCEDURE Vector_Field3
;  Vector_Field3 plots a 3-D vector field from three 3-D arrays.
;
;  SEE ALSO :  Vol_Marker
;
;  MANDATORY INPUTS :
;
;  vx :             A 3 dimensional array containing the X components
;                   of the vector field.
;
;  vy :             A 3-D array (with the same size as vx) containing
;                   the Y components of the vector field.
;
;  vz :             A 3-D array (with the same size as vx) containing
;                   the Z components of the vector field.
;
;  n_points :       The number of vectors to plot.
;
;  OPTIONAL INPUTS (Keywords) :
;
;  max_length :     A scalar value for the maximum plotted length of
;                   each vector. Each vector will range from zero to
;                   max_length.
;
;  vec_color :      A 3-D array (with the same dimensions as vx)
;                   containing the data for the vector colors.
;                   A vector plotted where vec_color is at its
;                   maximum will have the color "max_color", while a
;                   vector plotted at a location where vec_color is
;                   minimum will have the color "min_color".
;                   If "vec_color" is not supplied then the color of
;                   each vector will be determined by its length.
;
;  min_color :      The lowest color index to use when plotting.
;                   The color of each vector will range from min_color
;                   to max_color.
;
;  max_color :      The highest color index to use when plotting.
;
;  axis_color :     The color to use when plotting the axis.   Setting
;                   axis_color to -1 will suppress the axis.
;
;  mark_symbol :    Markers are plotted at the TAIL of each vector.
;                   The marker symbol number may range between 1 and
;                   7.   The default is 3 (period).
;
;  mark_color :     The color index to use when plotting the markers.
;
;  mark_size :      The marker size.
;
;  thick :          The line thickness (in pixels) to use when
;                   plotting vectors.
;
;=====================================================================
;
PRO Vector_Field3, vx, vy, vz, n_points, max_length=max_length, $
    vec_color=vec_color, min_color=min_color, max_color=max_color, $
    axis_color=axis_color, mark_symbol=mark_symbol, $
    mark_color=mark_color, mark_size=mark_size, thick=thick

sx = Size(vx)
sy = Size(vy)
sz = Size(vz)

IF (sx(0) NE 3) THEN BEGIN
   Print, "vx array not 3 dimensional."
   STOP
ENDIF
IF (sy(0) NE 3) THEN BEGIN
   Print, "vy array not 3 dimensional."
   STOP
ENDIF
IF (sz(0) NE 3) THEN BEGIN
   Print, "vz array not 3 dimensional."
   STOP
ENDIF
IF (((sx(1) LT 2) OR (sx(2) LT 2)) OR (sx(3) LT 2)) THEN BEGIN
   Print, "Volume dimensions must be >= 2"
   STOP
ENDIF
IF ((((sx(1) NE sy(1)) OR (sx(2) NE sy(2))) OR (sx(3) NE sy(3))) OR $
    (((sx(1) NE sz(1)) OR (sx(2) NE sz(2))) OR (sx(3) NE sz(3)))) THEN BEGIN
   Print, "vx, vy, and vz do not have the same dimensions."
   STOP
ENDIF

n_points = Long(n_points(0)) > Long(1)

px = Fix(Randomu(s, n_points) * sx(1)) < (sx(1) - 1)
py = Fix(Randomu(s, n_points) * sy(2)) < (sy(2) - 1)
pz = Fix(Randomu(s, n_points) * sz(3)) < (sz(3) - 1)

vecx = Invert(!P.T) # [1.0, 0.0, 0.0, 1.0]
vecy = Invert(!P.T) # [0.0, 1.0, 0.0, 1.0]
vecz = Invert(!P.T) # [0.0, 0.0, 1.0, 1.0]
h_error = 1000.0
vecx = (vecx * h_error) / ABS(vecx(3) * h_error)
vecy = (vecy * h_error) / ABS(vecy(3) * h_error)
vecz = (vecz * h_error) / ABS(vecz(3) * h_error)
veye = [vecx(2), vecy(2), vecz(2)]
mv = Max(ABS(veye))
dir_sort = Where(ABS(veye) GE mv)
IF (veye(dir_sort(0)) GE 0.0) THEN BEGIN
   CASE dir_sort(0) OF
      0: px = px(Sort(px))
      1: py = py(Sort(py))
      2: pz = pz(Sort(pz))
   ENDCASE
ENDIF ELSE BEGIN
   CASE dir_sort(0) OF
      0: px = Max(px) - px(Sort(px))
      1: py = Max(py) - py(Sort(py))
      2: pz = Max(pz) - pz(Sort(pz))
   ENDCASE
ENDELSE

axis_col = !D.N_Colors - 1
IF (N_Elements(axis_color) GT 0) THEN BEGIN
   axis_color = Fix(axis_color(0))
   axis_col = axis_color
ENDIF

dn = Float(Max([sx(1), sy(2), sz(3)]) - 1)
dn2 = dn / 2.0
xmid = Float(sx(1) - 1) / 2.0
ymid = Float(sy(2) - 1) / 2.0
zmid = Float(sz(3) - 1) / 2.0
!X.S = [(dn2 - xmid), 1.0] / dn
!Y.S = [(dn2 - ymid), 1.0] / dn
!Z.S = [(dn2 - zmid), 1.0] / dn
!X.Window = [0.0, Float(sx(1) - 1) / dn] + !X.S(0)
!Y.Window = [0.0, Float(sy(2) - 1) / dn] + !Y.S(0)
!Z.Window = [0.0, Float(sz(3) - 1) / dn] + !Z.S(0)
!X.Range = [0.0, (sx(1) - 1)]
!Y.Range = [0.0, (sy(2) - 1)]
!Z.Range = [0.0, (sz(3) - 1)]

xpos = 0.0
ypos = 0.0
zpos = 0.0
IF (vecx(2) LT 0.0) THEN xpos = Float(sx(1) - 1)
IF (vecy(2) LT 0.0) THEN ypos = Float(sy(2) - 1)
IF (vecz(2) LT 0.0) THEN zpos = Float(sz(3) - 1)
IF (axis_col GE 0) THEN BEGIN
   Axis, xpos, ypos, zpos, /Xaxis, Color=axis_col, /T3d
   Axis, xpos, ypos, zpos, /Yaxis, Color=axis_col, /T3d
   Axis, xpos, ypos, zpos, /Zaxis, Color=axis_col, /T3d
   Erase
   Axis, xpos, ypos, zpos, /Xaxis, Color=axis_col, /T3d
   Axis, xpos, ypos, zpos, /Yaxis, Color=axis_col, /T3d
   Axis, xpos, ypos, zpos, /Zaxis, Color=axis_col, /T3d
   Empty
ENDIF

ml = Max((vx * vx) + (vy * vy) + (vz * vz))
ml = Sqrt(Float(ml))
IF (ml EQ 0.0) THEN BEGIN
   Print, "Unable to plot vectors."
   Print, "The vx, vy, and vz arrays contain only zeroes."
   STOP
ENDIF
length = 1.0
max_len = ml
IF (N_Elements(max_length) GT 0) THEN BEGIN
   max_length = Float(max_length(0))
   max_len = max_length
   length = max_length / ml
ENDIF

IF (N_Elements(vec_color) GT 0) THEN BEGIN
   sc = Size(vec_color)
   IF (sc(0) NE 3) THEN BEGIN
      Print, "Vec_color array does not have 3 dimensions."
      STOP
   ENDIF
   IF (((sc(1) NE sx(1)) OR (sc(2) NE sx(2))) OR (sc(3) NE sx(3))) THEN BEGIN
      Print, "Vec_color array is not the same size as vx, vy, and vz."
      STOP
   ENDIF
ENDIF

min_col = 0
IF (N_Elements(min_color) GT 0) THEN BEGIN
   min_color = Fix(min_color(0)) > 0
   min_col = min_color
ENDIF
max_col = !D.N_Colors - 1
IF (N_Elements(max_color) GT 0) THEN BEGIN
   max_color = Fix(max_color(0)) > 0
   max_col = max_color
ENDIF

mark_sym = 3
IF (N_Elements(mark_symbol) GT 0) THEN BEGIN
   mark_symbol = Fix(mark_symbol(0))
   IF ((mark_symbol LT 1) OR (mark_symbol GT 7)) THEN BEGIN
      Print, "Marker symbol must be >= 1 and <= 7"
      STOP
   ENDIF
   mark_sym = mark_symbol
ENDIF
mark_col = !D.N_Colors - 1
IF (N_Elements(mark_color) GT 0) THEN BEGIN
   mark_color = Fix(mark_color(0)) > 0
   mark_col = mark_color
ENDIF
mark_sz = 1.0
IF (N_Elements(mark_size) GT 0) THEN BEGIN
   mark_size = Float(mark_size(0)) > 0.0
   mark_sz = mark_size
ENDIF
wide = 1
IF (N_Elements(thick) GT 0) THEN BEGIN
   thick = Fix(thick(0))
   wide = thick
ENDIF

IF (max_len NE 0.0) THEN BEGIN
   IF (N_Elements(vec_color) GT 0) THEN BEGIN
      min_vc = Float(Min(vec_color))
      max_vc = Float(Max(vec_color))
      col_fac = Float(max_col - min_col) / (Float(max_vc - min_vc) > 1.0)
      FOR i=0, (n_points - 1) DO BEGIN
         dx = length * vx(px(i), py(i), pz(i))
         dy = length * vy(px(i), py(i), pz(i))
         dz = length * vz(px(i), py(i), pz(i))
         x = [px(i), (px(i) + dx)]
         y = [py(i), (py(i) + dy)]
         z = [pz(i), (pz(i) + dz)]
         col_val = Float(vec_color(px(i), py(i), pz(i)))
         p_color = (col_fac * (col_val - min_vc)) + min_col
         Plots, x, y, z, Color=p_color, Thick=wide, /T3d
         Plots, x(0), y(0), z(0), Color=mark_col, Psym=mark_sym, Symsize=mark_sz, /T3d
      ENDFOR
   ENDIF ELSE BEGIN
      FOR i=0, (n_points - 1) DO BEGIN
         dx = length * vx(px(i), py(i), pz(i))
         dy = length * vy(px(i), py(i), pz(i))
         dz = length * vz(px(i), py(i), pz(i))
         x = [px(i), (px(i) + dx)]
         y = [py(i), (py(i) + dy)]
         z = [pz(i), (pz(i) + dz)]
         lv = Sqrt(Float((dx * dx) + (dy * dy) + (dz * dz)))
         p_color = ((lv / max_len) * max_col) > min_col
         Plots, x, y, z, Color=p_color, Thick=wide, /T3d
         Plots, x(0), y(0), z(0), Color=mark_col, Psym=mark_sym, Symsize=mark_sz, /T3d
      ENDFOR
   ENDELSE
ENDIF ELSE BEGIN
   FOR i=0, (n_points - 1) DO BEGIN
      x = [px(i), (px(i) + (length * vx(px(i), py(i), pz(i))))]
      y = [py(i), (py(i) + (length * vy(px(i), py(i), pz(i))))]
      z = [pz(i), (pz(i) + (length * vz(px(i), py(i), pz(i))))]
      lv = Sqrt((x * x) + (y * y) + (z * z))
      Plots, x, y, z, Color=max_col, Thick=wide, /T3d
      Plots, x(0), y(0), z(0), Color=mark_col, Psym=mark_sym, Symsize=mark_sz, /T3d
   ENDFOR
ENDELSE

END
