;=====================================================================
;                           PV-WAVE ARL
;
;                 PV-Wave Advanced Rendering Library
;                        11/90   D. Carr
;
;             (c) Copyright Precision Visuals, Inc., 1990
;                     Boulder, Colorado, U.S.A.
;
;                        All Rights Reserved.
;
;  This software is confidential information which is proprietary to
;  and a trade secret of Precision Visuals, Inc. Use, duplication or
;  disclosure is subject to the terms of an appropriate license
;  agreement.
;  ------------------------------------------------------------------
;  Use, duplication, or disclosure by the Government is subject to
;  restrictions set forth in paragraph (b)(3)(B) of the Rights in
;  Technical Data and Computer Software clause in DAR 7-104.9(a).
;
;  Contractor : Precision Visuals, Inc., Boulder, Colorado, U.S.A.
;  -------------------------------------------------------------------
;
;  Program Overview : FUNCTION Vol_Trans
;  Vol_Trans returns a 3 dimensional volume of data transformed by a
;  4x4 matrix.   The returned volume is scaled into the range of bytes.
;  For best results, the volume to transform should first be processed
;  using the "Vol_Pad" fuction.
;
;  SEE ALSO :  Vol_Pad   Vol_Rend
;
;  MANDATORY INPUTS :
;
;  volume :       The 3 dimensional volume of data to transform.
;
;  dim :          A scalar value specifying the X, Y, and Z dimensions
;                 of the transformed volume to return.   Normally,
;                 dim is the largest of the three dimensions of the
;                 original volume.   Generally, the original volume
;                 should "fit" inside the transformed volume.
;
;  trans :        The 4x4 transformation matrix to use for the
;                 transformation.   Trans is often the system viewing
;                 transformation matrix !P.T .
;
;=====================================================================
;
FUNCTION Vol_Trans, volume, dim, trans

sv = Size(volume)
IF (sv(0) NE 3) THEN BEGIN
   Print, "Volume array not 3 dimensional."
   STOP
ENDIF
IF (((sv(1) LT 2) OR (sv(2) LT 2)) OR (sv(3) LT 2)) THEN BEGIN
   Print, "Dimensions of volume must be >= 2"
   STOP
ENDIF
dim = Fix(dim(0))
IF (dim LT 2) THEN BEGIN
   Print, "Dimensions of transformed volume must be >= 2"
   STOP
ENDIF
st = Size(trans)
IF (st(0) NE 2) THEN BEGIN
   Print, "Invalid transformation matrix."
   STOP
END
IF ((st(1) NE 4) OR (st(2) NE 4)) THEN BEGIN
   Print, "Transformation matrix must be 4x4 ."
   STOP
ENDIF

vx = sv(1) - 1
vy = sv(2) - 1
vz = sv(3) - 1

volume_t = Rebin(Reform(volume(0:0, 0:0, 0:0), 1, 1, 1), dim, dim, dim)
dimm1 = dim - 1
dimm1f = Float(dimm1)
dimsq = dim * dim
dimsqm1 = dimsq - 1

du = Findgen(dim) / dimm1f
px = du # Replicate(1.0, dim)
py = Replicate(1.0, dim) # du
pz = Replicate(0.0, dimsq)
un = Replicate(1.0, dimsq)

trans = Invert(trans)
xind = Intarr(dimsq)
yind = Intarr(dimsq)
zind = Intarr(dimsq)
vec = Fltarr(dimsq, 4)
tvec = ([[px(*)], [py(*)], [pz(*)], [un(*)]])
tempi = Rebin(Reform(volume(0:0, 0:0, 0:0), 1, 1), dim, dim)

FOR i=0, dimm1 DO BEGIN
   tvec(*, 2) = Float(i) / dimm1f
   vec(*, *) = tvec # trans
   xind(*) = ((Fix(((vec(*, 0) / vec(*, 3)) - !X.S(0)) / !X.S(1))) > 0) < vx
   yind(*) = ((Fix(((vec(*, 1) / vec(*, 3)) - !Y.S(0)) / !Y.S(1))) > 0) < vy
   zind(*) = ((Fix(((vec(*, 2) / vec(*, 3)) - !Z.S(0)) / !Z.S(1))) > 0) < vz
   tempi(*) = volume([xind(*)], [yind(*)], [zind(*)])
   volume_t(*, *, i) = tempi(*, *)
ENDFOR

RETURN, volume_t
END
